
# Function used by both postscript and pdf devices
# Given a postscript file name and an output format,
# generate a file per page with the given format
psconvert <- function(filename, format, width, height) {
  if (charmatch(format, "pbm", nomatch=FALSE)) {
  # pstopnm is NOT part of netpbm -- needs gs!?
    system(paste("pstopnm -xsize ", width, " -ysize ", height,
                 " -pbm ", filename, sep=""),
           # Remove annoying messages from pstopnm
           ignore.stderr=TRUE)
  } else if (charmatch(format, "ps", nomatch=FALSE)) {
    # psselect unix-only?
    numpages <- as.numeric(system(paste("grep -c %%Page:", filename),
                                  intern=TRUE))
    for (i in 1:numpages) {
      system(paste("psselect -q ", i, " ",
                   filename, " > ", filename, 
                   i, ".ps", sep=""))
    }
  }
  else
    stop(paste("Unable to convert postscript to", format))
}

postscript.fns <-
  list(
       open=function(filebasename, suffix, width, height, ...) {
         postscript(file=paste(filebasename, suffix, sep=""), ...)
       },
       close=function(filebasename, ...) {
         dev.off()
       },
       convert=function(dir, filebasename, suffix,
         format, width, height, ...) {
         filename <- file.path(dir, paste(filebasename, suffix, sep=""))
         # Check for empty file and remove it
         # Can assume existence of grep on all platforms?
         if (length(system(paste("grep '%%Pages: 0'", filename), TRUE)) > 0)
           file.remove(filename)
         else {
           psconvert(filename, format, width, height)
         }
       },
       clean=function(dir, filebasename, suffix) {
         file.remove(file.path(dir, paste(filebasename, suffix, sep="")))
       })

x11.fns <-
  list(
       open=function(filebasename, suffix, width, height, ...) {
         png(file=paste(filebasename, suffix, "%d.png", sep=""),
             width=width, height=height,
             colortype="mono", ...)
       },
       close=function(filebasename, ...) {
         dev.off()
       },
       convert=function(dir, filebasename, suffix,
         format, width, height, ...) {
         filename <- paste(filebasename, suffix, sep="")
         # Get list of all files created with this base name
         files <- list.files(dir,
                             paste("^", filename, "*.png", sep=""),
                             full.names=TRUE)
         for (i in files) {
           # Check for empty file and remove it
           if (file.info(i)$size == 0)
             file.remove(i)
           else {
             if (charmatch(format, "pbm", nomatch=FALSE)) {
               # R produces a PNG image with
               # color_type=PNG_COLOR_TYPE_PALETTE or
               # color_type=PNG_COLOR_TYPE_RGB
               # This means that pngtopnm will never generate a pbm
               # (see pngtopnm.c in the pngtopnm source -- it will
               #  only convert to either pgm or ppm in this case)
               # So we have to convert the PNG to b&w first
               # This uses "convert" to do it, which is pretty certainly
               # NOT platform-independent.
               tempfilename <- file.path(dir, "tmp.png")
               system(paste("convert -monochrome ", i, " ", tempfilename))
               system(paste("pngtopnm ", tempfilename, " > ",
                            gsub(".png", "", i), ".pbm", sep=""),
                      # Remove annoying messages from pstopnm
                      ignore.stderr=TRUE)
               # May have generated a ppm or a pgm
               # If so, convert
               ppmfile <- paste(i, ".ppm", sep="")
               if (file.exists(ppmfile))
                 system(paste("ppmtopgm ", ppmfile, " > ",
                              i, ".pgm", sep=""))
               pgmfile <- paste(i, ".pgm", sep="")
               if (file.exists(pgmfile))
                 system(paste("pgmtopbm ", pgmfile, " > ",
                              i, ".pbm", sep=""))
             }
             else
               stop(paste("Unable to convert x11 to", format))
           }
         }
       },
       clean=function(dir, filebasename, suffix) {
         files <- list.files(dir,
                             paste("^", filebasename, suffix, "*.png", sep=""),
                             full.names=TRUE)
         for (i in files)
           file.remove(i)
         file.remove(file.path(dir, "tmp.png"))
       })

pdf.fns <-
  list(
       open=function(filebasename, suffix, width, height, ...) {
         pdf(file=paste(filebasename, suffix, sep=""), ...)
       },
       close=function(filebasename, ...) {
         dev.off()
       },
       convert=function(dir, filebasename, suffix,
         format, width, height, ...) {
         filename <- file.path(dir, paste(filebasename, suffix, sep=""))
         # Replace pdf file with ps version
         temppdfname <- file.path(dir, "tmp.pdf")
         temppsname <- file.path(dir, "tmp.ps")
         file.rename(filename, temppdfname)
         system(paste("pdftops", temppdfname, temppsname))
         file.rename(temppsname, filename)
         # Check for empty file and remove it
         # Can assume existence of grep on all platforms?
         if (length(system(paste("grep '%%Pages: 0'", filename), TRUE)) > 0)
           file.remove(filename)
         else {
           psconvert(filename, format, width, height)
         }
       },
       clean=function(dir, filebasename, suffix) {
         file.remove(file.path(dir, paste(filebasename, suffix, sep="")))
         file.remove(file.path(dir, "tmp.pdf"))
         file.remove(file.path(dir, "tmp.ps"))
       })

