"model.graphics" <-
function(funs=NULL,
         package=NULL,
         names=NULL,
         width=600,
         height=600,
         device=postscript,
         format="pbm",
         model.loc=".",
         verbose=FALSE,
         reset.rng=TRUE,
         # additional arguments to the device
         ...) {
  if (is.character(device)) {
    device.call <- get(device)
    dev.name <- device
  }
  else {
    device.call <- device
    dev.name <- deparse(substitute(device))
  }
  if (charmatch(dev.name, "postscript", nomatch=FALSE))
    device.fns <- postscript.fns
  else if (charmatch(dev.name, "x11", nomatch=FALSE))
    device.fns <- x11.fns
  else if (charmatch(dev.name, "pdf", nomatch=FALSE))
    device.fns <- pdf.fns
  else
    stop(paste("QC for the", dev.name, "device has not yet been implemented"))
  # width and height must be integer
  width <- as.integer(width)
  height <- as.integer(height)
  # model.loc must be character
  model.loc <- as.character(model.loc)
  # Must specify exactly one of funs or package
  if (is.null(funs)) {
    if (is.null(package))
      stop("Must specify one of `funs' or `package'")
    else {
      if (!is.character(package))
        package <- deparse(substitute(package))
      # Omit names beginning with %, [, ...
      # or containing <-
      funs <- ls(paste("package:", package, sep=""),
                 pattern="^[^[%><|(@$&=^~\-:?/{*+!].*[^<][^\-]")
    }
  }
  # funs must be character
  if (!is.character(funs))
    stop("`funs' must be a character vector")
  # names must be NULL or same length as funs
  if (!is.null(names)) {
    if (length(names) != length(funs))
      stop("`names' must be the same length as `funs'")
  }
  # If the model.loc directory doesn't exist, create it
  if (!file.exists(model.loc))
    if (!file.create(model.loc))
      stop(paste("Unable to create directory", model.loc))
  count <- 1
  for (i in funs) {
    if (verbose)
      cat("model.graphics:", i, "\n");
    # Generate the model output
    filebasename <- paste(if (is.null(names)) i else names[count],
                          "-", dev.name, "-",
                          width, "x", height, sep="")
    filefullname <- file.path(model.loc, filebasename)
    device.fns$open(filefullname, ".save.", width, height, ...)
    # Shut up example()'s warnings about "help file, but no examples"
    options(warn=-1)
    # Reset random number generator so that randomly-generated data
    # do not show up as differences
    if (reset.rng)
      set.seed(7654, "Wichmann-Hill")
    # Need to do something like rm(list = ls(all.names = TRUE))
    # to tidy up the environment between runs of example()
    # Wrap this in a try so that any "stop()"s in example code
    # do not chuck us out
    # Can't use example(i) because example() always does a substitute(topic)
    try(eval(call("example", i, echo=FALSE)))
    options(warn=0)
    device.fns$close(filefullname, ...)
    # Convert to bitmap format
    device.fns$convert(model.loc, filebasename, ".save.",
                       format, width, height, ...)
    count <- count + 1
  }
}
